From f67ea3a1a52eedd4d798e1937a66f5a929546044 Mon Sep 17 00:00:00 2001
From: "Robert A. Burks" <rburksdev@gmail.com>
Date: Mon, 11 Mar 2024 17:32:06 -0400
Subject: [PATCH 04/31] Add ert for variable watchers (bug#00001)

* test/src/data-tests.el (data-tests-watchers-alias-bug00001):
(data-tests-watchers-forward-bug00001):
Add ert for variable watcher alias and forwarded symbol bugs. (bug#00001)
---
 test/src/data-tests.el | 33 +++++++++++++++++++++++++++++++++
 1 file changed, 33 insertions(+)

diff --git a/test/src/data-tests.el b/test/src/data-tests.el
index 0778f8238ac..1d34dd3c4a9 100644
--- a/test/src/data-tests.el
+++ b/test/src/data-tests.el
@@ -610,6 +610,39 @@ data-tests-get-variable-watchers-inputs
     (should-error (get-variable-watchers bar2))
     (should-error (get-variable-watchers bar3))))
 
+;; BUG#00001 Test watcher when using `set-default' with an alias
+(ert-deftest data-tests-watchers-alias-bug00001 ()
+  (let* ((r1 nil)
+         (a1 (gensym))
+         (v1 (gensym))
+         (f1 (lambda (&rest args) (push args r1))))
+    (set v1 5)
+    (add-variable-watcher v1 f1)
+    (defvaralias a1 v1)
+    (set-default a1 100)
+    (should (equal (pop r1) `(,v1 100 set nil)))
+    (should (null  r1))))
+
+;; BUG#00001
+;; Test watchers of forwarded symbol for a particular double notification
+;; bug that arises from set_internal and set_default_internal each having
+;; cases in which they call the other.
+;; `right-margin-width' is an example of a DEFVAR_PER_BUFFER forwarded
+;; symbol.  In this case it shadows the default value.
+(ert-deftest data-tests-watchers-forward-bug00001 ()
+  (let* ((r1 nil)
+         (o1 (default-value 'right-margin-width))
+         (f1 (lambda (&rest args) (push args r1))))
+    (add-variable-watcher 'right-margin-width f1)
+    (with-temp-buffer
+      (let ((right-margin-width 50))
+        (set 'right-margin-width 20)))
+    (should (equal (pop r1) `(right-margin-width ,o1 unlet nil)))
+    (should (equal (pop r1) '(right-margin-width 20 set nil)))
+    (should (equal (pop r1) '(right-margin-width 50 let nil)))
+    (should (null  r1))
+    (remove-variable-watcher 'right-margin-width f1)))
+
 (ert-deftest data-tests-kill-all-local-variables () ;bug#30846
   (with-temp-buffer
     (setq-local data-tests-foo1 1)
-- 
2.34.1

